@@ -1,27 +1,10 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
 #include "config.h"
 
-#ifdef HAVE_FUSE3
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 30
-#endif
-#else
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-#endif
-
-#define _FILE_OFFSET_BITS 64
-
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <fuse.h>
 #include <inttypes.h>
 #include <libgen.h>
 #include <linux/magic.h>
@@ -46,8 +29,9 @@
 #include <unistd.h>
 #include <wait.h>
 
-#include "api_extensions.h"
 #include "bindings.h"
+
+#include "api_extensions.h"
 #include "cgroup_fuse.h"
 #include "cgroups/cgroup.h"
 #include "cgroups/cgroup_utils.h"
@@ -58,6 +42,8 @@
 
 static bool can_use_pidfd;
 static bool can_use_swap;
+static bool can_use_sys_cpu;
+static bool has_versioned_opts;
 
 static volatile sig_atomic_t reload_successful;
 
@@ -71,6 +57,16 @@ bool liblxcfs_can_use_swap(void)
 	return can_use_swap;
 }
 
+bool liblxcfs_can_use_sys_cpu(void)
+{
+	return can_use_sys_cpu;
+}
+
+bool liblxcfs_has_versioned_opts(void)
+{
+	return has_versioned_opts;
+}
+
 /* Define pivot_root() if missing from the C library */
 #ifndef HAVE_PIVOT_ROOT
 static int pivot_root(const char *new_root, const char *put_old)
@@ -609,7 +605,7 @@ static bool is_on_ramfs(void)
 	return false;
 }
 
-static int pivot_enter()
+static int pivot_enter(void)
 {
 	__do_close int oldroot = -EBADF, newroot = -EBADF;
 
@@ -652,7 +648,7 @@ static int pivot_enter()
 	return 0;
 }
 
-static int chroot_enter()
+static int chroot_enter(void)
 {
 	if (mount(ROOTDIR, "/", NULL, MS_REC | MS_BIND, NULL)) {
 		lxcfs_error("Failed to recursively bind-mount %s into /.", ROOTDIR);
@@ -899,8 +895,8 @@ static void __attribute__((constructor)) lxcfs_init(void)
 		lxcfs_info("Kernel does not support swap accounting");
 
 	lxcfs_info("api_extensions:");
-	for (i = 0; i < nr_api_extensions; i++)
-		lxcfs_info("- %s", api_extensions[i]);
+	for (size_t nr = 0; nr < nr_api_extensions; nr++)
+		lxcfs_info("- %s", api_extensions[nr]);
 
 	root_fd = open("/", O_PATH | O_CLOEXEC);
 	if (root_fd < 0)
@@ -929,3 +925,11 @@ static void __attribute__((destructor)) lxcfs_exit(void)
 	free_cpuview();
 	cgroup_exit(cgroup_ops);
 }
+
+void *lxcfs_fuse_init(struct fuse_conn_info *conn, void *data)
+{
+	struct fuse_context *fc = fuse_get_context();
+	can_use_sys_cpu = true;
+	has_versioned_opts = true;
+	return fc->private_data;
+}
