@@ -3,17 +3,9 @@
 #ifndef __LXCFS_BINDINGS_H
 #define __LXCFS_BINDINGS_H
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
+#include "config.h"
 
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-
-#define _FILE_OFFSET_BITS 64
-
-#include <fuse.h>
+#include <linux/types.h>
 #include <signal.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -22,8 +14,9 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "lxcfs_fuse.h"
+
 #include "cgroup_fuse.h"
-#include "config.h"
 #include "macro.h"
 #include "proc_cpuview.h"
 #include "proc_fuse.h"
@@ -85,13 +78,50 @@ struct lxcfs_opts {
 	bool swap_off;
 	bool use_pidfd;
 	bool use_cfs;
+	/*
+	 * Ideally we'd version by size but because of backwards compatability
+	 * and the use of bool instead of explicited __u32 and __u64 we can't.
+	 */
+	__u32 version;
 };
 
+typedef enum lxcfs_opt_t {
+	LXCFS_SWAP_ON	= 0,
+	LXCFS_PIDFD_ON	= 1,
+	LXCFS_CFS_ON	= 2,
+	LXCFS_OPTS_MAX	= LXCFS_CFS_ON,
+} lxcfs_opt_t;
+
+
 extern pid_t lookup_initpid_in_store(pid_t qpid);
 extern void prune_init_slice(char *cg);
 extern bool supports_pidfd(void);
 extern bool liblxcfs_functional(void);
 extern bool liblxcfs_can_use_swap(void);
+extern bool liblxcfs_can_use_sys_cpu(void);
+extern bool liblxcfs_has_versioned_opts(void);
+
+static inline bool lxcfs_has_opt(struct lxcfs_opts *opts, lxcfs_opt_t opt)
+{
+	if (!opts)
+		return false;
+
+	if (opt > LXCFS_OPTS_MAX)
+		return false;
+
+	switch (opt) {
+	case LXCFS_SWAP_ON:
+		if (!opts->swap_off)
+			return liblxcfs_can_use_swap();
+		return false;
+	case LXCFS_PIDFD_ON:
+		return opts->use_pidfd;
+	case LXCFS_CFS_ON:
+		return opts->use_cfs;
+	}
+
+	return false;
+}
 
 static inline int install_signal_handler(int signo,
 					 void (*handler)(int, siginfo_t *, void *))
@@ -120,4 +150,6 @@ static inline pid_t lxcfs_clone(int (*fn)(void *), void *arg, int flags)
 	return pid;
 }
 
+__visible extern void *lxcfs_fuse_init(struct fuse_conn_info *conn, void *data);
+
 #endif /* __LXCFS_BINDINGS_H */
