@@ -1,28 +1,10 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
 #include "config.h"
 
-#ifdef HAVE_FUSE3
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 30
-#endif
-#else
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-#endif
-
-#define _FILE_OFFSET_BITS 64
-
-#define __STDC_FORMAT_MACROS
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <fuse.h>
 #include <inttypes.h>
 #include <libgen.h>
 #include <pthread.h>
@@ -47,6 +29,8 @@
 #include <sys/sysinfo.h>
 #include <sys/vfs.h>
 
+#include "cgroup_fuse.h"
+
 #include "bindings.h"
 #include "cgroups/cgroup.h"
 #include "cgroups/cgroup_utils.h"
@@ -190,7 +174,7 @@ static bool is_child_cgroup(const char *controller, const char *cgroup,
 }
 
 /*
- * If pid is in /a/b/c, he may see that /a exists, but not /b or /a/c.
+ * If pid is in /a/b/c, they may see that /a exists, but not /b or /a/c.
  */
 static bool caller_may_see_dir(pid_t pid, const char *contrl, const char *cg)
 {
@@ -267,8 +251,8 @@ static char *get_next_cgroup_dir(const char *taskcg, const char *querycg)
 }
 
 /*
- * If pid is in /a/b/c/d, he may only act on things under cg=/a/b/c/d.
- * If pid is in /a, he may act on /a/b, but not on /b.
+ * If pid is in /a/b/c/d, they may only act on things under cg=/a/b/c/d.
+ * If pid is in /a, they may act on /a/b, but not on /b.
  * if the answer is false and nextcg is not NULL, then *nextcg will point
  * to a string containing the next cgroup directory under cg, which must be
  * freed by the caller.
@@ -357,7 +341,7 @@ static struct cgfs_files *cgfs_get_key(const char *controller,
  * pid's namespace.
  * Returns the mapped id, or -1 on error.
  */
-static unsigned int convert_id_to_ns(FILE *idfile, unsigned int in_id)
+static int convert_id_to_ns(FILE *idfile, unsigned int in_id)
 {
 	unsigned int nsuid,   // base id for a range in the idfile's namespace
 		     hostuid, // base id for a range in the caller's namespace
@@ -396,7 +380,7 @@ static unsigned int convert_id_to_ns(FILE *idfile, unsigned int in_id)
 
 /*
  * for is_privileged_over,
- * specify whether we require the calling uid to be root in his
+ * specify whether we require the calling uid to be root in their
  * namespace
  */
 #define NS_ROOT_REQD true
@@ -406,12 +390,13 @@ static unsigned int convert_id_to_ns(FILE *idfile, unsigned int in_id)
 
 static bool is_privileged_over(pid_t pid, uid_t uid, uid_t victim, bool req_ns_root)
 {
+	FILE *f;
 	char fpath[PROCLEN];
 	int ret;
 	bool answer = false;
 	uid_t nsuid;
 
-	if (victim == -1 || uid == -1)
+	if (victim == (uid_t)-1 || uid == (uid_t)-1)
 		return false;
 
 	/*
@@ -425,11 +410,12 @@ static bool is_privileged_over(pid_t pid, uid_t uid, uid_t victim, bool req_ns_r
 	ret = snprintf(fpath, PROCLEN, "/proc/%d/uid_map", pid);
 	if (ret < 0 || ret >= PROCLEN)
 		return false;
-	FILE *f = fopen(fpath, "re");
+
+	f = fopen(fpath, "re");
 	if (!f)
 		return false;
 
-	/* if caller's not root in his namespace, reject */
+	/* if caller's not root in their namespace, reject */
 	nsuid = convert_id_to_ns(f, uid);
 	if (nsuid)
 		goto out;
@@ -440,7 +426,7 @@ static bool is_privileged_over(pid_t pid, uid_t uid, uid_t victim, bool req_ns_r
 	 * will be sending requests where the vfs has converted
 	 */
 	nsuid = convert_id_to_ns(f, victim);
-	if (nsuid == -1)
+	if (nsuid == (uid_t)-1)
 		goto out;
 
 	answer = true;
@@ -955,7 +941,7 @@ __lxcfs_fuse_ops int cg_chmod(const char *path, mode_t mode)
 	/*
 	 * This being a fuse request, the uid and gid must be valid
 	 * in the caller's namespace.  So we can just check to make
-	 * sure that the caller is root in his uid, and privileged
+	 * sure that the caller is root in their uid, and privileged
 	 * over the file's current owner.
 	 */
 	if (!is_privileged_over(fc->pid, fc->uid, k->uid, NS_ROOT_OPT)) {
@@ -1068,7 +1054,7 @@ __lxcfs_fuse_ops int cg_chown(const char *path, uid_t uid, gid_t gid)
 	/*
 	 * This being a fuse request, the uid and gid must be valid
 	 * in the caller's namespace.  So we can just check to make
-	 * sure that the caller is root in his uid, and privileged
+	 * sure that the caller is root in their uid, and privileged
 	 * over the file's current owner.
 	 */
 	if (!is_privileged_over(fc->pid, fc->uid, k->uid, NS_ROOT_REQD)) {
@@ -1218,7 +1204,7 @@ static void pid_to_ns_wrapper(int sock, pid_t tpid)
 	char v;
 
 	ret = snprintf(fnam, sizeof(fnam), "/proc/%d/ns/pid", tpid);
-	if (ret < 0 || ret >= sizeof(fnam))
+	if (ret < 0 || (size_t)ret >= sizeof(fnam))
 		_exit(1);
 	newnsfd = open(fnam, O_RDONLY);
 	if (newnsfd < 0)
@@ -1361,7 +1347,8 @@ __lxcfs_fuse_ops int cg_read(const char *path, char *buf, size_t size,
 	struct file_info *f = INTTYPE_TO_PTR(fi->fh);
 	struct cgfs_files *k = NULL;
 	char *data = NULL;
-	int ret, s;
+	int ret;
+	size_t s;
 	bool r;
 
 	if (!liblxcfs_functional())
@@ -1414,7 +1401,7 @@ __lxcfs_fuse_ops int cg_read(const char *path, char *buf, size_t size,
 	if (s > size)
 		s = size;
 	memcpy(buf, data, s);
-	if (s > 0 && s < size && data[s-1] != '\n')
+	if ((s > 0) && (s < size) && (data[s - 1] != '\n'))
 		buf[s++] = '\n';
 
 	ret = s;
@@ -1555,7 +1542,7 @@ static void pid_from_ns_wrapper(int sock, pid_t tpid)
 	char v;
 
 	ret = snprintf(fnam, sizeof(fnam), "/proc/%d/ns/pid", tpid);
-	if (ret < 0 || ret >= sizeof(fnam))
+	if (ret < 0 || (size_t)ret >= sizeof(fnam))
 		_exit(1);
 	newnsfd = open(fnam, O_RDONLY);
 	if (newnsfd < 0)
@@ -1647,7 +1634,7 @@ static bool hostuid_to_ns(uid_t uid, pid_t pid, uid_t *answer)
 	*answer = convert_id_to_ns(f, uid);
 	fclose(f);
 
-	if (*answer == -1)
+	if (*answer == (uid_t)-1)
 		return false;
 	return true;
 }
@@ -1764,6 +1751,7 @@ static bool cgfs_set_value(const char *controller, const char *cgroup,
 	__do_free char *path = NULL;
 	int cfd;
 	size_t len;
+	ssize_t ret;
 
 	cfd = get_cgroup_fd_handle_named(controller);
 	if (cfd < 0)
@@ -1776,7 +1764,11 @@ static bool cgfs_set_value(const char *controller, const char *cgroup,
 		return false;
 
 	len = strlen(value);
-	return write_nointr(fd, value, len) == len;
+	ret = write_nointr(fd, value, len);
+	if (ret < 0)
+		return false;
+
+	return (size_t)ret == len;
 }
 
 __lxcfs_fuse_ops int cg_write(const char *path, const char *buf, size_t size,
@@ -1872,7 +1864,7 @@ static bool cgfs_iterate_cgroup(const char *controller, const char *cgroup,
 			continue;
 
 		ret = snprintf(pathname, sizeof(pathname), "%s/%s", path, dirent->d_name);
-		if (ret < 0 || ret >= sizeof(pathname)) {
+		if (ret < 0 || (size_t)ret >= sizeof(pathname)) {
 			lxcfs_error("Pathname too long under %s\n", path);
 			continue;
 		}
