@@ -3,22 +3,13 @@
 #ifndef __LXC_CGROUP_H
 #define __LXC_CGROUP_H
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-
-#define _FILE_OFFSET_BITS 64
+#include "config.h"
 
 #include <errno.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <sys/types.h>
 
-#include "../config.h"
 #include "../macro.h"
 #include "../memory_utils.h"
 
